#!/bin/bash
#
# appserver.sh
# Copyright (c) 2016, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved.
#
# appserver.sh      Starts the SAS Studio Application Server
#

# Uncomment the set -x to run in debug mode
# set -x

javaminmajor=1
javaminminor=8
oraclelatestpath="/usr/java/latest"
openjdkjre18path="/usr/lib/jvm/jre-1.8.0"
openjdkjrepath="/usr/lib/jvm/jre-openjdk"
jrepath="/usr/lib/jvm/jre"

# default configurable variables
deployid="viya"
instance="default"
productid="sasstudio"
pidfile=""
rootdir="/opt/sas/${deployid}"
logdir="/var/log/sas/${deployid}/${productid}/${instance}"
outfile="${logdir}/SASStudio.out"

logMessage()
{
    if [ -d "${logdir}" ]; then
        echo "$1" >> "${outfile}"
    else
        echo "$1"
    fi
}

checkVersion()
{
    local pathtojava=$1
    local result=1 # assume failure

    local version
    local major
    local minor

    # See if we were given a path to Java
    if [ -n "${pathtojava}" ] ; then
        logMessage "Checking version of ${pathtojava}."

        # Run java -version and capture the version number
        version=$("${pathtojava}" -version 2>&1 | head -n 1 | cut -d\" -f 2)
        if [ -n "${version}" ] ; then
            major=$(echo ${version} | cut -d'.' -f1)
            minor=$(echo ${version} | cut -d'.' -f2)
            if [ -n "${major}" -a -n "${minor}" ] ; then
                if [ \( "${major}" -eq "${javaminmajor}" \) -a \( "${minor}" -ge "${javaminminor}" \) ] ; then
                    logMessage "Version check passed."
                    result=0
                elif [ \( "${major}" -gt "${javaminmajor}" \) ] ; then
                    logMessage "Version check passed."
                    result=0
                else
                    logMessage "Version ${javaminmajor}.${javaminminor} or later required."
                    logMessage "Version ${version} detected."
                fi
            else
                logMessage "Error: Unable to parse major and minor version."
            fi
        else
            logMessage "Error: Unable to parse version version."
        fi
    else
        logMessage "Path to Java executable is required."
    fi

    return ${result};
}

checkExistsAndVersion()
{
    local pathtojava=$1
    local result=1 # assume failure

    # See if we were given a path to Java
    if [ -n "${pathtojava}" ] ; then
        # See if the Java executable exists
        if [ -x "${pathtojava}" ] ; then
            checkVersion "${pathtojava}"
            result=$?
        else
            logMessage "Java executable does not exist."
            logMessage "Java path: ${pathtojava}."
        fi
    else
        logMessage "Path to Java executable is required."
    fi

    return ${result}
}

die()
{
    logMessage "$1"
    exit 1
}

findJava()
{
    local searching=1
    
    if [ -n "${JAVA_SKIP_VALIDATION}" ] ; then
        searching=0
        logMessage "Skipping Java Validation."
    fi

    # Look at $SAS_JAVA_HOME
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for \$SAS_JAVA_HOME."
        if [ -n "${SAS_JAVA_HOME}" ] ; then
            checkExistsAndVersion "${SAS_JAVA_HOME}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Found Java using SAS_JAVA_HOME environment variable."
                export JAVA_HOME="${SAS_JAVA_HOME}"
            else
                die "Error: SAS_JAVA_HOME enviroment variable is invalid."
            fi
        else
            logMessage "\$SAS_JAVA_HOME is not set."
        fi
    fi

    # Look at $JAVA_HOME
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for \$JAVA_HOME"
        if [ -n "${JAVA_HOME}" ] ; then
            checkExistsAndVersion "${JAVA_HOME}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Found Java using JAVA_HOME environment variable."
            else
                die "Error: JAVA_HOME environment variable is invalid."
            fi
        else
            logMessage "\$JAVA_HOME is not set."
        fi
    fi

    # Look for Oracle JRE
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for Oracle JRE at ${oraclelatestpath}."
        if [ -e "${oraclelatestpath}" ] ; then
            checkExistsAndVersion "${oraclelatestpath}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Using Oracle JRE."
                export JAVA_HOME="${oraclelatestpath}"
            fi
        fi
    fi

    # Look for OpenJDK 1.8 JRE
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for OpenJDK JRE 1.8 at ${openjdkjre18path}."
        if [ -e "${openjdkjre18path}" ] ; then
            checkExistsAndVersion "${openjdkjre18path}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Using OpenJDK."
                export JAVA_HOME="${openjdkjre18path}"
            fi
        fi
    fi

    # Look for OpenJDK JRE
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for OpenJDK JRE at ${openjdkjrepath}."
        if [ -e "${openjdkjrepath}" ] ; then
            checkExistsAndVersion "${openjdkjrepath}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Using OpenJDK."
                export JAVA_HOME="${openjdkjrepath}"
            fi
        fi
    fi

    # Look for JRE
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for JRE at ${jrepath}."
        if [ -e "${jrepath}" ] ; then
            checkExistsAndVersion "${jrepath}/bin/java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Using OpenJDK."
                export JAVA_HOME="${jrepath}"
            fi
        fi
    fi

    # Look at $PATH
    if [ "$searching" -eq 1 ] ; then
        logMessage "Looking for Java in \$PATH"
        which java >/dev/null 2>&1
        if [ "$?" -eq 0 ] ; then
            logMessage "Java found in the path, checking the version."
            checkVersion "java"
            if [ "$?" -eq 0 ] ; then
                searching=0
                logMessage "Using Java from path."
            fi
        fi
    fi

    return ${searching}
}

while [ -n "$1" ]
do
  case "$1" in
    -p)
      shift
      pidfile="$1"
      ;;

    -r)
      shift
      rootdir="$1"
      ;;

    -l)
      shift
      logdir="$1"
      ;;

    -d)
      shift
      deployid="$1"
      ;;

    -i)
      shift
      instance="$1"
      ;;

     *)
      break  # don't recognize argument as expected opt, break and let something else consume those
      ;;
  esac
  shift  # discard the option's value
done

binhome="${rootdir}/home/SASStudio/bin"
confhome="${rootdir}/config/etc/${productid}/${instance}"
tmphome="${rootdir}/config/var/tmp/${productid}/${instance}"
command="${binhome}/sas.sasstudio.host"
outfile="${logdir}/SASStudio.out"
certframehome="${rootdir}/home/SASSecurityCertificateFramework"
certframeconf="${rootdir}/config/etc/SASSecurityCertificateFramework"

consulconf="${rootdir}/config/consul.conf"
deploymodsappserver="${confhome}/appserver_deployment.sh"
usermodsappserver="${confhome}/appserver_usermods.sh"

# look for sas-java and sas-java-services* sysconfig options
# the java_bin local variable will be sourced from sas-java
for sysconfigfile in ${rootdir}/config/etc/sysconfig/sas-javaesntl/{sas-java,sas-java-services*}
do
    [ -f $sysconfigfile ] && source $sysconfigfile
done

# Override ENV
[ -f ${consulconf} ] && . ${consulconf}
[ -f ${deploymodsappserver} ] && . ${deploymodsappserver}
[ -f ${usermodsappserver} ] && . ${usermodsappserver}

logMessage " "
logMessage "Start SAS Studio Application Server."

# Look for a version of Java we can use
findJava
if [ "$?" -ne 0 ] ; then
    die "Error: Unable to find Java."
fi

# Setup Java properties
SAS_SASSTUDIO_HOST_OPTS="${SAS_SASSTUDIO_HOST_OPTS} -Dsasstudio.appserver.logdir=${logdir} -Dsasstudio.appserver.confdir=${confhome} -Dsasstudio.deployment.deployid=${deployid} -Dsasstudio.deployment.productid=${productid} -Dsasstudio.deployment.instanceid=${instance} -Djava.io.tmpdir=${tmphome} -Dcertframe.home=${certframehome} -Dcertframe.confdir=${certframeconf} -Dsasstudio.appserver.consul.baseurl=${CONSUL_HTTP_ADDR} -Djava.security.egd=file:/dev/urandom"
export SAS_SASSTUDIO_HOST_OPTS

# Start the process
$command >>"${outfile}" 2>&1 &

# Save the PID
pid=$!
if [ -n "$pidfile" ]; then
    echo "$pid" > "$pidfile"
fi


